From: David Howells <dhowells@redhat.com>
To: torvalds@transmeta.com
Cc: linux-kernel@vger.kernel.org
Subject: [PATCH] anti-cross-CPU printk/oops interleaving
Date: Fri, 04 Jul 2003 10:11:18 +0100 [thread overview]
Message-ID: <27637.1057309878@warthog.warthog> (raw)
Hi Linus,
This patch prevents an oops on one CPU being interleaved char-by-char with
printks being performed on other CPUs, thus rendering them actually readable
under those circumstances.
David
diff -ur linux-2.5.74/kernel/printk.c linux-2.5.74-auto/kernel/printk.c
--- linux-2.5.74/kernel/printk.c 2003-07-03 15:37:24.000000000 +0100
+++ linux-2.5.74-auto/kernel/printk.c 2003-07-03 15:41:15.000000000 +0100
@@ -378,6 +378,9 @@
logged_chars++;
}
+/* cpu currently holding logbuf_lock */
+static volatile int printk_cpu = -1;
+
/*
* This is printk. It can be called from any context. We want it to work.
*
@@ -400,8 +403,9 @@
static char printk_buf[1024];
static int log_level_unknown = 1;
- if (oops_in_progress) {
- /* If a crash is occurring, make sure we can't deadlock */
+ if (oops_in_progress && printk_cpu == smp_processor_id()) {
+ /* If a crash is occurring during printk() on this CPU,
+ * make sure we can't deadlock */
spin_lock_init(&logbuf_lock);
/* And make sure that we print immediately */
init_MUTEX(&console_sem);
@@ -409,6 +413,7 @@
/* This stops the holder of console_sem just where we want him */
spin_lock_irqsave(&logbuf_lock, flags);
+ printk_cpu = smp_processor_id();
/* Emit the output into the temporary buffer */
va_start(args, fmt);
reply other threads:[~2003-07-04 8:56 UTC|newest]
Thread overview: [no followups] expand[flat|nested] mbox.gz Atom feed
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=27637.1057309878@warthog.warthog \
--to=dhowells@redhat.com \
--cc=linux-kernel@vger.kernel.org \
--cc=torvalds@transmeta.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox